! @file test_ieee_emulation.f90
! @brief test ieee_arithmetic module implementation
!
! Copyright  (C)  2012  Thomas Jahns <jahns@dkrz.de>
!
! @version 1.0
! Keywords: IEEE 754 floating-point arithmetics
! @author Thomas Jahns <jahns@dkrz.de>
! Maintainer: Thomas Jahns <jahns@dkrz.de>
! URL: https://www.dkrz.de/redmine/projects/show/scales-ppm
!
! Redistribution and use in source and binary forms, with or without
! modification, are  permitted provided that the following conditions are
! met:
!
! Redistributions of source code must retain the above copyright notice,
! this list of conditions and the following disclaimer.
!
! Redistributions in binary form must reproduce the above copyright
! notice, this list of conditions and the following disclaimer in the
! documentation and/or other materials provided with the distribution.
!
! Neither the name of the DKRZ GmbH nor the names of its contributors
! may be used to endorse or promote products derived from this software
! without specific prior written permission.
!
! THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
! IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
! TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
! PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
! OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
! EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
! PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
! PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
! LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
! NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
! SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
!
PROGRAM test_ieee_emulation
  USE ppm_base, ONLY: assertion
  USE ppm_std_type_kinds, ONLY: sp, dp
  USE ieee_arithmetic, ONLY: ieee_value, ieee_negative_inf, &
       ieee_quiet_nan, ieee_copy_sign, &
       ieee_negative_normal, ieee_negative_denormal, ieee_negative_zero, &
       ieee_positive_zero, ieee_positive_denormal, ieee_positive_normal, &
       ieee_positive_inf, ieee_is_normal
  IMPLICIT NONE
  REAL(sp) :: vs
  REAL(dp) :: vd
  CALL assertion(.NOT. &
       LOGICAL(ieee_is_normal(ieee_value(1.0_sp, ieee_quiet_nan))), &
       __FILE__, &
       __LINE__, 'quiet nan is normal?')
  vs = ieee_value(1.0_sp, ieee_negative_inf)
  CALL assertion(vs < 0.0_sp .AND. .NOT. ieee_is_normal(vs), &
       __FILE__, &
       __LINE__, 'negative infinity is normal or >= 0.0 ?')
  vs = ieee_value(1.0_sp, ieee_negative_normal)
  CALL assertion(vs < 0.0_sp .AND. ieee_is_normal(vs), &
       __FILE__, &
       __LINE__, 'negative normal is not normal or >= 0.0 ?')
  vs = ieee_value(1.0_sp, ieee_negative_denormal)
  CALL assertion(ieee_copy_sign(1.0_sp, vs) == -1.0_sp &
       .AND. .NOT. ieee_is_normal(vs), &
       __FILE__, &
       __LINE__, 'negative denormal is normal or >= 0.0 ?')
  vs = ieee_value(1.0_sp, ieee_negative_zero)
  CALL assertion(vs == 0.0_sp .AND. ieee_is_normal(vs), &
       __FILE__, &
       __LINE__, 'negative zero is not normal or /= 0.0 ?')
  vs = ieee_value(1.0_sp, ieee_positive_zero)
  CALL assertion(vs == 0.0_sp .AND. ieee_is_normal(vs), &
       __FILE__, &
       __LINE__, 'negative zero is not normal or /= 0.0 ?')
  vs = ieee_value(1.0_sp, ieee_positive_denormal)
  CALL assertion(ieee_copy_sign(1.0_sp, vs) == 1.0_sp &
       .AND. .NOT. ieee_is_normal(vs), &
       __FILE__, &
       __LINE__, 'positive denormal is normal or <= 0.0 ?')
  vs = ieee_value(1.0_sp, ieee_positive_normal)
  CALL assertion(vs > 0.0_sp .AND. ieee_is_normal(vs), &
       __FILE__, &
       __LINE__, 'positive normal is not normal or <= 0.0 ?')
  vs = ieee_value(1.0_sp, ieee_positive_inf)
  CALL assertion(vs > 0.0_sp .AND. .NOT. ieee_is_normal(vs), &
       __FILE__, &
       __LINE__, 'positive infinity is normal or <= 0.0 ?')
  CALL assertion(.NOT. &
       LOGICAL(ieee_is_normal(ieee_value(1.0_dp, ieee_quiet_nan))), &
       __FILE__, &
       __LINE__, 'quiet nan is normal?')
  vd = ieee_value(1.0_dp, ieee_negative_inf)
  CALL assertion(vd < 0.0_dp .AND. .NOT. ieee_is_normal(vd), &
       __FILE__, &
       __LINE__, 'negative infinity is normal or >= 0.0 ?')
  vd = ieee_value(1.0_dp, ieee_negative_normal)
  CALL assertion(vd < 0.0_dp .AND. ieee_is_normal(vd), &
       __FILE__, &
       __LINE__, 'negative normal is not normal or >= 0.0 ?')
  vd = ieee_value(1.0_dp, ieee_negative_denormal)
  CALL assertion(ieee_copy_sign(1.0_dp, vd) == -1.0_dp &
       .AND. .NOT. ieee_is_normal(vd), &
       __FILE__, &
       __LINE__, 'negative denormal is normal or >= 0.0 ?')
  vd = ieee_value(1.0_dp, ieee_negative_zero)
  CALL assertion(vd == 0.0_dp .AND. ieee_is_normal(vd), &
       __FILE__, &
       __LINE__, 'negative zero is not normal or /= 0.0 ?')
  vd = ieee_value(1.0_dp, ieee_positive_zero)
  CALL assertion(vd == 0.0_dp .AND. ieee_is_normal(vd), &
       __FILE__, &
       __LINE__, 'negative zero is not normal or /= 0.0 ?')
  vd = ieee_value(1.0_dp, ieee_positive_denormal)
  CALL assertion(ieee_copy_sign(1.0_dp, vd) == 1.0_dp &
       .AND. .NOT. ieee_is_normal(vd), &
       __FILE__, &
       __LINE__, 'positive denormal is normal or <= 0.0 ?')
  vd = ieee_value(1.0_dp, ieee_positive_normal)
  CALL assertion(vd > 0.0_dp .AND. ieee_is_normal(vd), &
       __FILE__, &
       __LINE__, 'positive normal is not normal or <= 0.0 ?')
  vd = ieee_value(1.0_dp, ieee_positive_inf)
  CALL assertion(vd > 0.0_dp .AND. .NOT. ieee_is_normal(vd), &
       __FILE__, &
       __LINE__, 'positive infinity is normal or <= 0.0 ?')
END PROGRAM test_ieee_emulation
!
! Local Variables:
! license-project-url: "https://www.dkrz.de/redmine/projects/show/scales-ppm"
! license-markup: "doxygen"
! license-default: "bsd"
! End:
!
